import java.util.*;

public class Population extends Observable {
    
  private int          individuenzahl;
  private static int   codelnge;
  private int          mutationsrate;
  private Individuum[] population;
  private IDatenGA     grunddaten;
  private Paar         paar;

  public Population (int n, int c, int m, IDatenGA g) {
    individuenzahl = n; population = new Individuum[n];
    mutationsrate = m; codelnge = c;
    grunddaten = g; belegeZufllig();
    } // Ende Konstruktor

  public void belegeZufllig() {
    for (int k = 0; k < individuenzahl; k++) {
      Individuum code = new Individuum(codelnge);
      grunddaten.initZufllig(code);
      population[k] = code;
      } // Ende for
    setChanged(); 
    notifyObservers();
    } // Ende belege zufllig

  public int getIndividuenzahl()
    {return individuenzahl;}

  public IDatenGA getGrunddaten()
    {return grunddaten;}

  public int getCodelnge()
    {return codelnge;}

  public void setMutationsrate (int m)
    {mutationsrate = m;}

  public int getMutationsrate()
    {return mutationsrate;}

  public Individuum getIndividuum (int nr)
    {return population[nr];}

  public void setIndividuum (int nr, Individuum i)
      {population[nr] = i;}

  public void nchsteGeneration() {
    Individuum[] nchsteGen = new Individuum[individuenzahl];
    for (int k = 0; k < individuenzahl; k = k + 2) {
      Individuum i1 = liesZufllig();
      Individuum i2 = liesZufllig();
      Paar paar = grunddaten.kreuzung(i1, i2);
      nchsteGen[k] = paar.getErstes();
      int zufall = (int) Math.random() * 100;
      if (zufall <= mutationsrate) 
        grunddaten.mutiere(nchsteGen[k]);
      if (k + 1 < individuenzahl) {
        nchsteGen[k+1] = paar.getZweites(); 
        zufall = (int) Math.random() * 100;
        if (zufall <= mutationsrate) 
          grunddaten.mutiere(nchsteGen[k + 1]);
        } // Ende if
      } // Ende for
    population = nchsteGen;
    setChanged(); 
    notifyObservers();
    } // Ende nchste Generation

  public Individuum bestes() {
    int nr = 0;
    int max = grunddaten.fitnesswert(population[nr]);
    Individuum bestesgeneration = population[nr];
    for (int k = 1; k < individuenzahl; k++) {
      int wert = grunddaten.fitnesswert(population[k]);
      if (grunddaten.istBesser(population[k], population[nr])) {
        max = wert; nr = k;
        bestesgeneration = population[k];
        } // Ende if
      } // Ende for
      return bestesgeneration;
      } // Ende bestes

  private Individuum liesZufllig() {
    int nr1 = (int) Math.random() * individuenzahl;
    int nr2 = (int) Math.random() * individuenzahl;
    if (grunddaten.istBesser(population[nr1], population[nr2]))
       return population[nr1];
    else 
      return population[nr2];
    } // Ende lies zufllig

  public static void main (String[] xy) {
    DatenGA grunddaten = new DatenGA(codelnge);
    Population paar = new Population(10, 12, 2, grunddaten);
    for (int i = 0; i < paar.individuenzahl; i++) {
      Individuum c = paar.getIndividuum(i);
      System.out.print(c);
      System.out.println("  " + grunddaten.fitnesswert(c));
      } // Ende for
    } // Ende main

  } // Ende Population
